// Header file for BitObj

// TDIBobject   provides basic loading, saving and displaying
//              functionality 

// TDIBobjras   extends the functionality to include basic raster
//              operations such as setting and retrieving pixels
//              and whole scan-lines, manipulating the palette,
//              and a constructor for a blank DIBobjras.



#ifndef H_DIBOBJ       // Guards against compiling twice
#define H_DIBOBJ

#include <filedial.h>
#include <string.h>

#include "..\fastcadr\dibobjrc.h"


_CLASSDEF (TDIBobject)

class TDIBobject
{
protected:
 PBITMAPINFO pbmi;

 HBITMAP BitsHandle;
 DWORD bytesLine;
 HPALETTE hpal;

 HPALETTE CreateHpal();

public:
 TDIBobject(LPSTR filename, UINT &errorval, UINT flags=DO_NORMAL);

 TDIBobject() {};

 void Save(LPSTR Name, UINT &errorval);

 WORD GetWidth() { return pbmi->bmiHeader.biWidth; };
 WORD GetHeight() { return pbmi->bmiHeader.biHeight; };

 int GetXdpi(void) { return int(pbmi->bmiHeader.biXPelsPerMeter*25.4/1000 +0.5); };
 int GetYdpi(void) { return int(pbmi->bmiHeader.biYPelsPerMeter*25.4/1000 +0.5); };

 // Really want these two in basic class?
 void SetXdpi(int dpi) { pbmi->bmiHeader.biXPelsPerMeter=int(dpi*(1000/25.4) +0.5); };
 void SetYdpi(int dpi) { pbmi->bmiHeader.biYPelsPerMeter=int(dpi*(1000/25.4) +0.5); };

 HPALETTE GetHpal() { return hpal; };

 void DIBtoDevice(HDC aDC, int XDest, int YDest, int cx, int cy, int XSrc, int YSrc);
 void StretchDIBtoDevice(HDC aDC, int XDest, int YDest, int cxDest, int cyDest, int XSrc, int YSrc, int cxSrc, int cySrc);
 HBITMAP CreateDDB(HDC aDC);

 ~TDIBobject();
};


_CLASSDEF(TDIBobjras)

class TDIBobjras : public TDIBobject
{
protected:
 long bits;

public:
 TDIBobjras(LPSTR filename, UINT &errorval)
       : TDIBobject(filename, errorval, DO_RASTER) {};

 TDIBobjras(WORD width, WORD height, WORD numColors, RGBQUAD &colors, UINT &errorval);

 void Lock(void) { bits=(long)GlobalLock(BitsHandle); };
 void Unlock(void) { GlobalUnlock(BitsHandle); };

 void SetLine(int L, BYTE *pixels);
 void GetLine(int L, BYTE *pixels);

 WORD GetNumColours() { return pbmi->bmiHeader.biClrUsed; };
 RGBQUAD GetRGBfromPalette(WORD PalIndex) { return (pbmi->bmiColors[PalIndex]); };

 RGBQUAD GetPixelRGB(WORD x, WORD y);
 WORD GetPixelPaletteIndex(WORD x, WORD y);
 inline void SetPixelPaletteIndex(WORD x, WORD y, WORD PalIndex);
/*
 void LineDraw(int x1, int y1, int x2, int y2, WORD PalIndex);
 void TriangleFill(POINT *corner, WORD PalIndex);
 void Wipe();
*/
};


class TBMPFileDialog : public TFileDialog {
public:
    TBMPFileDialog(PTWindowsObject AParent, int ResourceId,
                   LPSTR AFilePath, PTModule AModule = NULL)
     : TFileDialog(AParent, ResourceId, AFilePath, AModule)
       { strcpy(Extension, ".bmp"); }
};


#endif