// Program to collect digitised pictures from the EPP
// parallel port

// Collects ONE FIELD at a time from NEW CIRCUIT featuring
// MSM518221 frame buffer IC.

// (It appears the raw data-transfer rate cannot exceed 500kB/s.)


// (C) W.A.Steer 1998

#include <owl.h>
#include <bwcc.h>
#include <dos.h>

#include "..\fastcadr\dibobj17.h"

#include "video.h"

#define VID_PIX 840       // 840 // 906 max
#define VID_LINES 304

#define APP_NAME "W.A.S. Video Digitiser 2.0a"

_CLASSDEF (TVideoWin)

class TVideoWin : public TWindow
{
private:
 PTDIBobjras image;

public:
 TVideoWin(PTWindowsObject AParent, LPSTR ATitle);
 ~TVideoWin();


 virtual UINT WMPaletteChanged(RTMessage Msg)
  = [ WM_FIRST + WM_PALETTECHANGED ];

 virtual UINT WMQueryNewPalette(RTMessage Msg)
  = [ WM_FIRST + WM_QUERYNEWPALETTE ];

 void Paint(HDC PaintDC, PAINTSTRUCT& );


// virtual void WMLButtonDown(RTMessage Msg)
//  = [ WM_FIRST + WM_LBUTTONDOWN ];
 virtual void WMGrab(RTMessage Msg)
    = [ CM_FIRST + CM_GRAB ];


 virtual void CMFileSaveAs(RTMessage Msg)
  = [ CM_FIRST + CM_FILESAVE ];
};


TVideoWin::TVideoWin(PTWindowsObject AParent, LPSTR ATitle)
 : TWindow(AParent, ATitle)
{
 AssignMenu("MENU_1");

 RGBQUAD colors[256];

 int sg;

 for (int g=0; g<256; g++)
 {
//  sg=(g*8)/5; if (sg>255) sg=255;
//  sg=(g*3)/2; if (sg>255) sg=255;
sg=g;
  colors[g].rgbRed=colors[g].rgbGreen=colors[g].rgbBlue=sg;
  colors[g].rgbReserved=0;
 }


 UINT errorval;

 image = new TDIBobjras(VID_PIX,VID_LINES, 256, *colors, errorval);
}

TVideoWin::~TVideoWin()
{
 delete image;
}

UINT TVideoWin::WMPaletteChanged(RTMessage Msg)
{
 if ((HWND)Msg.WParam != HWindow)
 {
  return WMQueryNewPalette(Msg);
 }
 else return 0;
}


UINT TVideoWin::WMQueryNewPalette(RTMessage)
{
 UINT i=0;

 if (!IsIconic(HWindow))
 {
  HPALETTE hpal,OldPalette;
  HDC hdc;
  hdc=GetDC(HWindow);

  hpal=image->GetHpal();

  OldPalette=SelectPalette(hdc, hpal, FALSE);
  i=RealizePalette(hdc);
  SelectPalette(hdc, OldPalette, FALSE);

  ReleaseDC(HWindow,hdc);

  if (i>0) InvalidateRect(HWindow,NULL,TRUE);
 }
 return i;
}


void TVideoWin::Paint(HDC PaintDC, PAINTSTRUCT& )
{
 HPALETTE hpal,oldpal;

 hpal=image->GetHpal();
 oldpal=SelectPalette(PaintDC,hpal,False);
 UnrealizeObject(hpal);
 RealizePalette(PaintDC);

// image->StretchDIBtoDevice(PaintDC,0,0,384,image->GetHeight(), 0,0,image->GetWidth(),image->GetHeight());
 image->DIBtoDevice(PaintDC,0,0,image->GetWidth(),image->GetHeight(), 0,0);

 SelectPalette(PaintDC,oldpal,False);
}

void TVideoWin::WMGrab(RTMessage)
{
 const int portbase=0x278;

 const int status=portbase+1;
 const int control=portbase+2;
 const int eppdata=portbase+4;

 int scanbuf[VID_PIX/2];

 WORD counter, maxwait=32768u;


 // Initialise port settings
 outportb(portbase,0);
 outportb(control,4+32);

 HCURSOR oldCursor = SetCursor(LoadCursor(0,IDC_WAIT));

 image->Lock();

// Field buffer position???
 outportb(control, 32);
 inport(eppdata);
 outportb(control, 36);

// for (int dummy=0; dummy<2048; dummy++) inport(eppdata);
// int count=0; do { count++; } while ((inport(eppdata)!=0) && (count<VID_PIX));

 for(int line=0; line<VID_LINES; line++)
 {
  // Get the data
  for(register int pixel=0; pixel<(VID_PIX/2); pixel++)
   scanbuf[pixel]=inport(eppdata);

  image->SetLine(VID_LINES-1-line, (BYTE *)scanbuf);
 }

 image->Unlock();

 SetCursor(oldCursor);

 InvalidateRect(HWindow, NULL, FALSE);
 SendMessage(HWindow, WM_PAINT,0,0);
}

void TVideoWin::CMFileSaveAs(RTMessage)
{
 char TempName [MAXPATH];

 if (GetApplication()->ExecDialog( new TBMPFileDialog
  (this, SD_FILESAVE, strcpy(TempName, "*.bmp"))) == IDOK )
 {
  SetWindowText(HWindow,"Saving picture...");

  UINT errorval;

  image->Save(TempName, errorval);

  if (errorval!=DOE_SUCCESS)
  {
   char errmsg[DO_MAXERRLEN];

   LoadString(GetApplication()->hInstance, errorval, errmsg, sizeof(errmsg));

   BWCCMessageBox(HWindow,errmsg,"Error saving picture", MB_OK | MB_ICONSTOP);
  }

  SetWindowText(HWindow,APP_NAME);
 }

}

// ----------------  TMyApp  ----------------

class TMyApp : public TApplication
{
public:
  TMyApp(LPSTR AName, HINSTANCE hInstance, HINSTANCE hPrevInstance,
    LPSTR lpCmdLine, int nCmdShow)
    : TApplication(AName, hInstance, hPrevInstance, lpCmdLine, nCmdShow) {};

  virtual void InitMainWindow();
};


void TMyApp::InitMainWindow()
{
 (void)BWCCGetVersion();    // Make sure BWCC gets initialized.
 MainWindow = new TVideoWin(NULL, Name);
}


int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
  LPSTR lpCmdLine, int nCmdShow)
{
 TMyApp MyApp(APP_NAME, hInstance, hPrevInstance,
               lpCmdLine, nCmdShow);

 // _InitEasyWin();

 MyApp.Run();
 return MyApp.Status;
}
